#| ---------------------------------------------------------------------------

  This file is part of NuRV (client sample code)
  Copyright (C) 2018-2021 Fondazione Bruno Kessler

  Author: Chun Tian (binghe) <ctian@fbk.eu>
-----------------------------------------------------------------------------|#

(in-package :cl-user)

(defun name-components (names)
  (mapcar #'(lambda (name) (CosNaming:NameComponent :id name   ;; Represents a node in a name
						    :kind "")) ;; Unused, can contain comments
	  names))

(defparameter *monitor-name* (name-components '("NuRV" "Monitor" "Service")))

(defvar *client-orb*   nil) ; ORB
(defvar *name-service* nil) ; NS
(defvar *service*      nil) ; Monitor:Service instance

(defun set-name-service (&optional string)
  (corba:set-pluggable-module-details "NameService" :ior-string string))

;; not used
(defun set-interface-repository (string)
  (corba:set-pluggable-module-details "InterfaceRepository" :ior-string string))

(defun get-name-service (orb)
  (let ((ref (op:resolve_initial_references orb "NameService")))
    (when ref
      (op:narrow 'CosNaming:NamingContext ref))))

(defun resolve-object (name)
  (unless *name-service*
    (warn "No name service found")
    (return-from resolve-object nil))
  (handler-case
      (op:resolve *name-service* name)
    (CosNaming:NamingContext/NotFound nil)))

(defun setup (ior)
  (setq *client-orb* (op:orb_init nil "LispWorks ORB"))
  (set-name-service ior)
  (setq *name-service* (get-name-service *client-orb*))
  (setq *service*
	(op:narrow 'Monitor:MonitorService (resolve-object *monitor-name*))))

(defgeneric monitor (index))
(defmethod monitor ((index integer))
  (corba:any :any-typecode corba:_tc_long :any-value index))

(defmethod monitor ((index string))
  (corba:any :any-typecode corba:_tc_string :any-value index))

;; Usage:
;;
;; (op:heartbeat *service* (monitor 0) "p & q") ; 0 is the id of an LTL property
;; (op:heartbeat *service* (monitor "p0") "!p") ; "p0" is the name of an LTL property
;; (op:reset *service* (monitor 0) nil)         ; nil means soft reset (hard_p = false)
